import { IDancingLinkX } from '@algorithm.ts/dlx';

/**
 * Sudoku constraints.
 */
declare enum SudokuConstraint {
    SLOT = 0,
    ROW = 1,
    COL = 2,
    SUB = 3
}
interface ISudokuSolverOptions {
    /**
     * Size of the child puzzle matrix (sqrt of original puzzle size)
     */
    readonly childMatrixSize: number;
}
declare class SudokuSolver {
    readonly SUDOKU_SIZE_SQRT: number;
    readonly SUDOKU_SIZE: number;
    readonly SUDOKU_SIZE_SQUARE: number;
    readonly DL_TOTAL_COLUMNS: number;
    protected readonly constraints: number[];
    protected readonly segmentCodeMap: ReadonlyArray<number>;
    protected readonly dlx: IDancingLinkX;
    constructor(options: ISudokuSolverOptions);
    /**
     * Try to solve the given Sudoku problem, if there is a solution, fill the
     * solution into the incoming `solution` board.
     *
     * In particular, you can use the same two-dimensional array to pass in as the
     * `puzzle` and `solution` parameters respectively. If there is a solution,
     * the solution will still be filled into the passed `solution` array.
     *
     * If the `solution` parameter is not passed in, the solution will be
     * discarded, which is equivalent to only judging whether there is a solution.
     *
     * @param puzzle
     * @param solution
     * @returns Whether there is a solution
     * @public
     */
    solve(puzzle: ReadonlyArray<number[]>, solution: number[][] | null): boolean;
}

/**
 * Sudoku game board.
 */
declare type ISudokuBoard = number[][];
/**
 * A sudoku game data.
 */
interface ISudokuData {
    /**
     * Sudoku puzzle.
     */
    puzzle: ISudokuBoard;
    /**
     * Solution of the sudoku puzzle.
     */
    solution: ISudokuBoard;
}

interface ISudokuCreatorOptions {
    /**
     * Size of the child puzzle matrix (sqrt of original puzzle size)
     */
    readonly childMatrixSize: number;
    /**
     * The difficulty to solve the puzzle.
     * @default 0.2
     */
    difficulty?: number;
}
declare class SudokuCreator {
    readonly SUDOKU_SIZE_SQRT: number;
    readonly SUDOKU_SIZE: number;
    readonly SUDOKU_SIZE_SQUARE: number;
    protected readonly solver: SudokuSolver;
    protected readonly segmentCodeMap: ReadonlyArray<number>;
    protected readonly gridCodes: number[];
    protected readonly candidates: number[];
    protected readonly visitedNums: boolean[];
    protected readonly tmpBoard: ISudokuBoard;
    protected difficulty: number;
    constructor(options: ISudokuCreatorOptions);
    /**
     * Create a sudoku game data.
     * @param _difficulty
     * @returns
     */
    createSudoku(_difficulty?: number): ISudokuData;
    /**
     * Create a full-filled sudoku data.
     * @returns
     */
    protected createSolution(): number[][];
    /**
     * Create a sudoku puzzle that has only one unique solution.
     * @param solution
     * @returns
     */
    protected createPuzzle(solution: Readonly<ISudokuBoard>): ISudokuBoard;
    /**
     * Calculate which numbers can be filled in a specified grid (r,c) in the
     * Sudoku board.
     *
     * For the sake of saving memory space, the number candidates are stored in
     * this.candidates, and the function only returns the number of candidates.
     *
     * @param board
     * @param r
     * @param c
     */
    protected calcCandidates(board: Readonly<ISudokuBoard>, r: number, c: number): number;
    /**
     * Resolve the value of difficulty.
     * @param _difficulty
     * @returns
     */
    protected resolveDifficulty(_difficulty: number): number;
}

/**
 * Create a sudoku board (two-dimensional array).
 * @param SUDOKU_SIZE
 */
declare function createSudokuBoard(SUDOKU_SIZE: number): ISudokuBoard;
/**
 * Fill the sudoku board with the given value.
 * @param board
 * @param v
 * @param SUDOKU_SIZE
 */
declare function fillSudokuBoard(board: ISudokuBoard, v: number, SUDOKU_SIZE?: number): void;
/**
 * Copy sudoku board from `src` to `dst`.
 * @param src
 * @param dst
 * @param SUDOKU_SIZE
 */
declare function copySudokuBoard(src: Readonly<ISudokuBoard>, dst: ISudokuBoard, SUDOKU_SIZE?: number): void;
/**
 * Check if the solution of the given Sudoku game is a valid solution.
 *
 * @param solution
 * @param SUDOKU_SIZE_SQRT
 * @returns
 */
declare function checkSudokuSolution(solution: ISudokuBoard, SUDOKU_SIZE_SQRT: number): boolean;
/**
 * segmentCodeMap 是一个映射表，用于快速计算某个格子所在行在其所在列上的第几个子
 * 方阵中（或者所在列在其所在行上的第几个子方阵中）
 *
 * segmentCodeMap is a mapping table for quickly located the sub-matrix of a grid.
 *
 * For example:
 *
 *    In a 9x9 sudoku board, a grid (x, y) at the sub-matrix
 *    `(Math.floor(x / 3), Math.floor(y / 3))`. If use segmentNo to represent the
 *    position, it is `(segmentNo[x], segmentNo[y])`
 *
 * @param SUDOKU_SIZE
 */
declare function createSegmentCodeMap(SUDOKU_SIZE_SQRT: number): number[];

export { ISudokuBoard, ISudokuCreatorOptions, ISudokuData, ISudokuSolverOptions, SudokuConstraint, SudokuCreator, SudokuSolver, checkSudokuSolution, copySudokuBoard, createSegmentCodeMap, createSudokuBoard, fillSudokuBoard };
